// (C) Copyright 2015-2019 NuoDB, Inc.  All Rights Reserved.

#pragma once

#include "SQLException.h"

namespace NuoDB
{

class BatchUpdateException : public SQLException
{
public:
    /**
     * @brief   Get the update count for a given statement executed in the batch
     *          update. The NuoDB C++ driver will continue to process the
     *          remaining batch commands when one of the commands fails to
     *          execute properly.
     *
     * @param   The 0-indexed index into the array of commands in the batch for
     *          which the update count should be returned
     *
     * @return  A 32-bit integer equal to one of the following for the
     *          (0-indexed) nth batch command executed:
     *          <ul>
     *          <li>an update count</li>
     *          <li>NuoDB::SUCCESS_NO_INFO indicating that the command executed
     *          successfully but the number of rows affected is unknown</li>
     *          <li>NuoDB::EXECUTE_FAILED to indicate that the command failed to
     *          execute successfully</li>
     *          </ul>
     */
    virtual int getUpdateCount(int n) const = 0;

    /**
     * @brief   Get the array of update counts for the batch update. The NuoDB
     *          C++ driver will continue to process the remaining batch commands
     *          when one of the commands fails to execute properly.
     *
     * @return  An array of 32-bit integers equal to one of the following for
     *          each of the batch commands executed:
     *          <ul>
     *          <li>an update count</li>
     *          <li>NuoDB::SUCCESS_NO_INFO indicating that the command executed
     *          successfully but the number of rows affected is unknown</li>
     *          <li>NuoDB::EXECUTE_FAILED to indicate that the command failed to
     *          execute successfully</li>
     *          </ul>
     */
    virtual int* getUpdateCounts() const = 0;

    /**
     * @brief   Get the error message for a given statement executed in the batch
     *          update. The NuoDB C++ driver will continue to process the
     *          remaining batch commands when one of the commands fails to
     *          execute properly.  If the command executed successfully, then
     *          this returns NULL.
     *
     * @param   The 0-indexed index into the array of commands in the batch for
     *          which the error message should be returned
     *
     * @return  A char* pointer to the error message triggered by the (0-indexed)
     *          nth batch command executed or a pointer to an empty string if
     *          the command executed successfully.
     */
    virtual const char* getText(int n) const = 0;

    /**
     * @brief   Get the error message for the first statement executed in a batch
     *          that caused an error. The NuoDB C++ driver will continue to
     *          process the remaining batch commands when one of the commands
     *          fails to execute properly.  If all commands executed
     *          successfully, then this returns NULL.
     *
     * @return  A char* pointer to the first error message triggered by a
     *          batch command or a pointer to an empty string if all commands
     *          executed successfully.
     */
    virtual const char* getText() const = 0;

    /**
     * @brief   Get the SQL code for a given statement executed in the batch
     *          update. The NuoDB C++ driver will continue to process the
     *          remaining batch commands when one of the commands fails to
     *          execute properly.  If the command executed successfully, then
     *          this returns 0. The SQL code returned is defined in
     *          NuoDB::NuoDBSqlConstants.
     *
     * @param   The 0-indexed index into the array of commands in the batch for
     *          which the SQL code should be returned
     *
     * @return  A 32-bit integer representing the SQL code of the error
     *          triggered by the (0-indexed) nth batch command executed or 0 if
     *          the command executed successfully.
     */
    virtual int getSqlcode(int n) const = 0;

    /**
     * @brief   Get the SQL code for the first statement executed in a batch
     *          update that caused an error. The NuoDB C++ driver will continue
     *          to process the remaining batch commands when one of the commands
     *          fails to execute properly.  If no command in the batch failed,
     *          then this returns 0. The SQL code returned is defined in
     *          NuoDB::NuoDBSqlConstants.
     *
     * @return  A 32-bit integer representing the SQL code of the first error
     *          triggered by a batch command executed or 0 if all
     *          commands executed successfully.
     */
    virtual int getSqlcode() const = 0;

    /**
     * @brief   Get the SQL state as defined by the SQL standard for a given
     *          statement executed in the batch update. The NuoDB C++ driver
     *          will continue to process the remaining batch commands when one of
     *          the commands fails to execute properly.  If the command executed
     *          successfully, then this returns an empty string.
     *          (Note: Not yet implemented...)
     *
     * @param   The 0-indexed index into the array of commands in the batch for
     *          which the SQL state should be returned
     *
     * @return  The SQL state associated with the error triggered by the
     *          (0-indexed) nth batch command executed or an empty string if the
     *          command executed successfully.
     */
    virtual const char* getSQLState(int n) const = 0;

    /**
     * @brief   Get the SQL state as defined by the SQL standard for the first
     *          statement executed in a batch update that caused an error. The
     *          NuoDB C++ driver will continue to process the remaining batch
     *          commands when one of the commands fails to execute properly.  If
     *          all commands executed successfully, then this returns an empty
     *          string.
     *          (Note: Not yet implemented...)
     *
     * @return  The SQL state associated with the error triggered by the
     *          (0-indexed) nth batch command executed or an empty string if the
     *          command executed successfully.
     */
    virtual const char* getSQLState() const = 0;

    /**
     * @brief   The number of commands executed (successfully or unsuccessfully)
     *          by the batch operation.
     *
     * @return  The number of commands executed in the associated batch operation.
     */
    virtual int getBatchCount() const = 0;

    /**
     * @brief   Get the stack trace for the throwing of this BatchUpdateException
     *
     * @param   buffer  pointer to a char* buffer to which the stack trace will
     *                  be written. The caller is responsible for allocating and
     *                  deleting this buffer.
     * @param   len     length of the buffer. If the stack trace is longer than
     *                  len, it will be truncated to fit into the buffer.
     *
     * @return  A pointer to the filled in buffer
     */
    virtual char* getTrace(char* buffer, size_t len) const = 0;

};
}
