/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

#include "NuoRemote/Bytes.h"
#include "NuoRemote/PreparedStatement.h"
#include "NuoRemote/SqlExports.h"

namespace NuoDB
{
struct Bytes;
class Date;
class Time;
class Timestamp;
class Blob;
class Clob;

static const int CALLABLESTATEMENT_VERSION  = 1;


/**
 * @brief This class is used to execute SQL stored procdures.
 *
 * The result parameter, if used, must be registered as an OUT parameter. The other
 * parameters can be used for input, output or both. Parameters are referred to
 * sequentially, by number, with the first parameter being 1.
 *
 * call <procedure-name>[(<arg1>,<arg2>, ...)]
 *
 * IN parameter values are set using the set methods inherited from
 * PreparedStatement. The type of all OUT parameters must be registered prior to
 * executing the stored procedure; their values are retrieved after execution
 * via the get methods provided here.
 *
 * A CallableStatement can return only one ResultSet object.
 */

class CallableStatement : public PreparedStatement
{
public:
    /**
     * @brief   Registers the OUT parameter in ordinal position parameterIndex
     *          to the type sqlType.
     *
     * All OUT parameters must be registered before a stored procedure is executed.
     *
     * The type specified by sqlType for an OUT parameter determines the
     * type that must be used in the get method to read the value of that
     * parameter.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and
     *                          so on
     * @param   sqlType         the type code defined by NuoDB::SqlType. If the
     *                          parameter is of type NUODB_NUMERIC or NUODB_DECIMAL,
     *                          the version of registerOutParameter that accepts
     *                          a scale value should be used.
     */
    virtual void        registerOutParameter(int parameterIndex, int sqlType) = 0;

    /**
     * @brief   Registers the OUT parameter in ordinal position parameterIndex
     *          to the type sqlType.
     *
     * All OUT parameters must be registered before a stored procedure is executed.
     *
     * The type specified by sqlType for an OUT parameter determines the
     * type that must be used in the get method to read the value of that
     * parameter.
     *
     * This version of registerOutParameter should be used when the parameter is
     * of type NUODB_NUMERIC or NUODB_DECIMAL.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     * @param   sqlType         the type code defined by NuoDB::SqlType. If the
     *                          parameter is of type NUODB_NUMERIC or NUODB_DECIMAL,
     *                          the version of registerOutParameter that accepts
     *                          a scale value should be used.
     * @param   scale           the desired number of digits to the right of the
     *                          decimal point. It must be greater than or equal
     *                          to zero.
     */
    /* c-api-suffix=with_scale */
    virtual void        registerOutParameter(int parameterIndex, int sqlType, int scale) = 0;

    /**
     * @brief   Retrieves whether the last OUT parameter read had the value of
     *          SQL NULL.
     *
     * @return  true if the value was NULL and false otherwise
     */
    virtual bool        wasNull() = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a const char* in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the string at position parameterIndex. If the value is SQL NULL,
     *          return null.
     *
     * @throw   SQLException
     */
    virtual const char* getString(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a bool in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the boolean parameter at position parameterIndex. If the value is
     *          SQL NULL, return false.
     *
     * @throw   SQLException
     */
    virtual bool        getBoolean(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a char in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the char parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual char        getByte(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a short in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the short parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual short       getShort(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a int32_t in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the int32_t parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual int32_t     getInt(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a int64_t in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the int64_t parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual int64_t     getLong(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a float in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the float parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual float       getFloat(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as a double in the C++ programming language.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the double parameter at position parameterIndex; if the value is
     *          SQL NULL, then return 0.
     *
     * @throw   SQLException
     */
    virtual double      getDouble(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated
     *          parameter as an object of type Bytes.
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  the Bytes parameter at position parameterIndex; if the value is
     *          SQL NULL, then return a Byte structure with a NULL pointer inside.
     *
     * @throw   SQLException
     */
    virtual Bytes       getBytes(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated parameter as a pointer to
     *          an object of type Date.
     *
     * Note: The user is responsible for freeing the resources allocated in the
     * returned Date object by calling Date::release().
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  a pointer to the Date parameter at position parameterIndex; if
     *          the value is SQL NULL, then return a Date with value 1970-01-01
     *
     * @throw   SQLException
     */
    virtual Date*       getDate(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated parameter as a pointer to
     *          an object of type Time.
     *
     * Note: The user is responsible for freeing the resources allocated in the
     * returned Time object by calling Time::release().
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  a pointer to the Time parameter at position parameterIndex; if the
     *          value is SQL NULL, then return a Time with value 00:00:00
     *
     * @throw   SQLException
     */
    virtual Time*       getTime(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated parameter as a pointer to
     *          an object of type Timestamp.
     *
     * Note: The user is responsible for freeing the resources allocated in the
     * returned Timestamp object by calling Timestamp::release().
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  a pointer to the Timestamp parameter at position parameterIndex;
     *          if the value is SQL NULL, then return a Timestamp with value
     *          1970-01-01 00:00:00
     *
     * @throw   SQLException
     */
    virtual Timestamp*  getTimestamp(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated parameter as a pointer to
     *          an object of type Blob.
     *
     * Note: The user is responsbile for releasing the Blob via Blob::release().
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  a pointer to the Blob parameter at position parameterIndex; if the
     *          value is SQL NULL, returns an empty Blob
     *
     * @throw   SQLException
     */
    virtual Blob*       getBlob(int parameterIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated parameter as a pointer to
     *          an object of type Clob.
     *
     * Note: The user is responsbile for releasing the Blob via Blob::release().
     *
     * @param   parameterIndex  the first parameter is 1, the second is 2, and so on
     *
     * @return  a pointer to the Clob parameter at position parameterIndex; if the
     *          value is SQL NULL, returns an empty Clob
     *
     * @throw   SQLException
     */
    virtual Clob*       getClob(int parameterIndex) = 0;

};
} // end namespace NuoDB
