/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

namespace NuoDB
{

/**
 * @brief Representation of a Character Large Object (CLOB) value to be 
 * stored as a column value in a row of a table.
 *
 * The representation in C++ of an SQL CLOB value. An SQL CLOB is a built-in data 
 * type that stores a Character Large Object as a column value in a row of a database 
 * table. 
 */
class Clob
{
protected:
    virtual ~Clob() {}

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the 
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /** 
     * @brief   Return number of bytes in the CLOB value specified by the CLOB 
     *          object
     * 
     * @return  the length in bytes of the CLOB
     */
    virtual int     length() = 0; 

    /** 
     * @brief   Return number of characters in the CLOB value specified by the 
     *          CLOB object
     * 
     * @return  the length in characters of the CLOB
     */
    virtual int     getNumberCharacters() = 0;

    /** 
     * @brief   Get the characters stored in the CLOB, starting at the specified
     *          offset,  and assign them to the space allocated and pointed to 
     *          by the buffer pointer
     *
     * Only copy length bytes to the buffer array.
     *
     * @param   offset  The offset at which to start copying.
     * @param   length  The number of bytes to copy into the buffer array
     * @param   buffer  A pointer to an array allocated with length bytes
     */
    virtual void    getChars(int offset, int length, char* buffer) = 0;

    /**
     * @brief   Copy length bytes from the designated buffer into this Clob 
     *          object
     *
     * @param   length  the number of bytes to copy
     * @param   buffer  the array of characters from which to initialize this Clob
     */
    virtual void    setChars(int length, const char* buffer) = 0;

    /** 
     * @brief   Append length bytes from the designed buffer to the end of this 
     *          Clob object
     *
     * @param   length  the number of bytes to copy
     * @param   buffer  the array of characters from which to append this Clob
     */
    virtual void    appendChars(int length, const char* buffer) = 0;
};
} // end of namespace NuoDB
