/* Copyright 2013-2015  NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

struct tm;

namespace NuoDB
{

/**
 * @brief   C++ class type equivalent to SQL DATE
 *
 * This class will allow you to manipulate this Date object that represents
 * the SQL DATE object stored in the database.
 *
 * See also NuoDB::SqlDate.
 */
class Date
{
protected:
    virtual ~Date() = 0;

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the 
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /**
     * @brief   Return the value of the corresponding SQL DATE as seconds
     *
     * The seconds returned will be UTC seconds since the common (UNIX) epoch, 
     * Jan 1, 1970.
     *
     * @return  the number of seconds in the SQL DATE
     */
    virtual int64_t getSeconds() = 0;   

    /**
     * @brief   Set the Date object with the specified number of seconds
     *
     * @param   seconds     a 64-bit integer specifying the number of seconds 
     *                      since the common (UNIX) epoch, January 1, 1970
     */
    virtual void    setSeconds(int64_t seconds) = 0;

    /**
     * @brief   Return the value of the corresponding SQL DATE as milliseconds
     *
     * The milliseconds returned will be UTC milliseconds since the common 
     * (UNIX) epoch, Jan 1, 1970.
     *
     * @return  the number of milliseconds in the SQL DATE
     */
    virtual int64_t getMilliseconds() = 0;

    /**
     * @brief   Set the Date object with the specified number of milliseconds
     *
     * @param   milliseconds    a 64-bit integer specifying the number of 
     *                          milliseconds since the common (UNIX) epoch, 
     *                          January 1, 1970
     */
    virtual void    setMilliseconds(int64_t milliseconds) = 0;
};
} // end of namespace NuoDB
