/* Copyright 2013-2015  NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

#include "NuoRemote/SqlExports.h"
#include "NuoRemote/Statement.h"

namespace NuoDB
{
class ParameterMetaData;
class ResultSet;
class Date;
class Time;
class Timestamp;
class Blob;
class Clob;
class BigDecimal;
class ResultSetMetaData;

static const int PREPAREDSTATEMENT_VERSION  = 1;

/**
 * @brief   A class that represents a precompiled SQL statement.
 *
 * An SQL statement is precompiled and stored in a PreparedStatement object.
 * This object can then be used to efficiently execute this statement multiple
 * times.
 *
 * Note: The setter methods (setShort(), setString(), and so on) for setting IN
 * parameter values specify types that are compatible with the defined SQL
 * type of the input parameter. For instance, if the IN parameter has SQL type
 * INTEGER, then the method setInt() should be used. Type conversions will be
 * attempted.
 *
 */
class PreparedStatement : public virtual Statement
{
protected:
    virtual ~PreparedStatement() {}

public:

    /**
     * @brief   Execute the prepared statement.
     *
     * The execute() method returns a boolean to indicate the form of the first
     * result. You must call either the method getResultSet() or getUpdateCount()
     * to retrieve the result.
     *
     * If autoCommit is not set on the Connection, then the statement is not
     * committed until Connection::commit() is called.
     *
     * @return  true if the first result is a ResultSet object; false if the
     *          first result is an update count or there is no result
     *
     * @throw   SQLException
     */
    virtual bool execute() = 0;

    /**
     * @brief   Executes the SQL query in this PreparedStatement object and
     *          returns the ResultSet object generated by the query.
     *
     * If autoCommit is not set on the Connection, then the statement is not
     * committed until Connection::commit() is called.
     *
     * @return  a ResultSet object that contains the data produced by the query;
     *          never null
     *
     * @throw   SQLException
     */
    virtual ResultSet* executeQuery() = 0;

    /**
     * @brief   Executes the SQL statement in this PreparedStatement object
     *
     * Using this form of execute, the SQL statement is typically an SQL Data
     * Manipulation Language (DML) statement, such as INSERT, UPDATE or DELETE;
     * or an SQL statement that returns nothing, such as a DDL statement.
     *
     * Call this only once per statement.
     *
     * @return  the number of objects updated by the statement or -1 if the
     *          return value is a result set
     *
     * @throw   SQLException
     */
    virtual int executeUpdate() = 0;

    /**
     * @brief   Sets the designated parameter to the given const char* value.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   string  the value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setString(int index, const char* string) = 0;

    /**
     * @brief   Sets the designated parameter to the given const char* value with
     *          specified length.
     *
     * The string argument is not assumed to be null terminated. Only the first
     * length chars will be used.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   string  the value to which we will set the parameter
     * @param   length  the legnth of the specified string
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_length */
    virtual void    setString(int index, const char* string, size_t length) = 0;

    /**
     * @brief   Sets the designated parameter to the given char value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   the one-byte value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setByte(int index, char value) = 0;

    /**
     * @brief   Sets the designated parameter to the given short value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value  the short value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setShort(int index, short value) = 0;

    /**
     * @brief   Sets the designated parameter to the given 32-bit integer value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value  the 32-bit integer value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setInt(int index, int32_t value) = 0;

    /**
     * @brief   Sets the designated parameter to the given 64-bit integer value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value  the 64-bit integer value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setLong(int index, int64_t value) = 0;

    /**
     * @brief   Sets the designated parameter to the value specified in the given
     *          array of bytes
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   length  the number of bytes to copy from the bytes parameter
     * @param   bytes   the array of bytes to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setBytes(int index, int length, const void* bytes) = 0;

    /**
     * @brief   Sets the designated parameter to the given float
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   the float value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setFloat(int index, float value) = 0;

    /**
     * @brief   Sets the designated parameter to the given double-precision value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   the double-precision value to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void    setDouble(int index, double value) = 0;

    /**
     * @brief   Sets the designated parameter to SQL NULL.
     *
     * Note: You must specify the parameter's SQL type.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   type    the type code as defined in NuoDB::SqlType
     *
     * @throw   SQLException
     */
    virtual void    setNull(int index, int type) = 0;

    /**
     * @brief   Sets the designated parameter to the given Date object
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a Date object to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setDate(int index, Date* value) = 0;

    /**
     * @brief   Sets the designated parameter to the given Time object
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a Time object to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setTime(int index, Time* value) = 0;

    /**
     * @brief   Sets the designated parameter to the given Timestamp object
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a Timestamp object to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setTimestamp(int index, Timestamp* value) = 0;

    /**
     * @brief   Sets the designated parameter to the given Blob value
     *
     * The caller is responsible for the memory (by calling release()) of the
     * Blob object after calling setBlob().
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a Blob object to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setBlob(int index, Blob* value) = 0;

    /**
     * @brief   Sets the designated parameter to the given Clob value
     *
     * The caller is responsible for the memory (by calling release()) of the
     * Clob object after calling setClob().
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a Clob object to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setClob(int index, Clob* value) = 0;

    /**
     * @brief   Retrieves the number, types and properties of this
     *          PreparedStatement object's parameters.
     *
     * @return  a pointer to a ParameterMetaData object
     *
     * @throw   SQLException
     */
    virtual ParameterMetaData*  getParameterMetaData() = 0;

    /* not implemented */
    virtual int                 objectVersion() = 0;

    /**
     * @brief   Clears the current parameter values immediately.
     *
     * In general, parameter values remain in force for repeated use of a
     * statement. Setting a parameter value automatically deletes and frees its
     * previous value. However, in some cases it is useful to immediately release
     * the resources used by the current parameter values; this can be done by
     * calling the method clearParameters().
     */
    virtual void                clearParameters() = 0;

    /*
     * @brief   Adds a set of parameters to this PreparedStatement object's
     *          batch of commands.
     */
    virtual void                addBatch() = 0;

    /**
     * @brief   Sets the designated parameter to the given BigDecimal value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a pointer to a BigDecimal which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void                setBigDecimal(int index, BigDecimal* value) = 0;

    /* not implemented */
    /* c-api-suffix=with_scale */
    virtual void    setInt(int index, int32_t value, int32_t scale) = 0;

    /* not implemented */
    /* c-api-suffix=with_scale */
    virtual void    setLong(int index, int64_t value, int32_t scale) = 0;

    /**
     * @brief   Sets the designated parameter to the given boolean value
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     * @param   value   a boolean to which we will set the parameter
     *
     * @throw   SQLException
     */
    virtual void setBoolean(int index, bool value) = 0;

    /**
     * @brief   Retrieves the number, types and properties of the
     *          ResultSet of this PreparedStatement.
     *
     * @return  a pointer to a ResultSetMetaData object if the PreparedStatement
                is a query. Otherwise, returns NULL.

     *
     * @throw   SQLException
     */
    virtual ResultSetMetaData*  getMetaData() = 0;
};
}                   // end namespace NuoDB
