/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

#include "NuoRemote/SqlExports.h"

namespace NuoDB
{

/**
 * @brief   Represents a persistent set of properties, which can be loaded from,
 *          or saved to, a stream. Each key and value pair are strings.
 */
class Properties
{
protected:
    virtual ~Properties() {}

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

public:
    /**
     * @brief   Return the value of the property specified by name
     *
     * If the property is not set, return the specified defaultValue.
     *
     * @param   name            The name of the property
     * @param   defaultValue    The default value to return if the property
     *                          specified by name is not set; can be NULL
     *
     * @return  The property's value or the default value if the property is not
     *          set
     */
    virtual const char* findValue(const char* name, const char* defaultValue) = 0;

    /**
     * @brief   Set the property specified by name to the given value
     *
     * @param   name    The name of the property
     * @param   value   The new value for property
     */
    virtual void        putValue(const char* name, const char* value) = 0;

    /**
     * @brief   Set the property specified by name to the given value
     *
     * Given that name and value may not be null terminated, specify length
     * nameLength and valueLength.
     *
     * @param   name        The name of the property
     * @param   nameLength  The length of the name property
     * @param   value       The new value for the property
     * @param   valueLength The length of the given value
     */
    /* c-api-suffix=with_length */
    virtual void        putValue(const char* name, int nameLength, const char* value, int valueLength) = 0;

    /**
     * @brief   Return the number of properties set in this Properties object
     *
     * @return  the number of properties set in this object
     */
    virtual int         getCount() = 0;

    /**
     * @brief   Get the name of the property at the given index
     *
     * @param   index   0-based index into the Properties object
     *
     * @return  the name of the property
     */
    virtual const char* getName(int index) = 0;

    /**
     * @brief   Get the name of the property's value at the given index
     *
     * @param   index   0-based index into the Properties object
     *
     * @return  the value
     */
    virtual const char* getValue(int index) = 0;
};
}
