/* Copyright 2013 NuoDB, Inc. All rights reserved */
#pragma once

namespace NuoDB
{
static const int RESULTSETMETADATA_VERSION  = 1;

/**
 * @brief   Enum defining constant types used by the C++ API
 */
enum SqlType
{
    NUOSQL_NULL = 0,            /**< Represents SQL value NULL (0) */
    NUOSQL_BIT = -7,            /**< Type code representing the SQL type BIT (-7) */
    NUOSQL_TINYINT = -6,        /**< Type code representing the SQL type BYTE (-6) */
    NUOSQL_SMALLINT = 5,        /**< Type code representing the SQL type SMALLINT (5) */
    NUOSQL_INTEGER = 4,         /**< Type code representing the SQL type INTEGER (4) */
    NUOSQL_BIGINT = -5,         /**< Type code representing the SQL type BIGINT (-5) */
    NUOSQL_FLOAT = 6,           /**< Type code representing the SQL type FLOAT (6) */
    NUOSQL_DOUBLE = 8,          /**< Type code representing the SQL type DOUBLE (8) */
    NUOSQL_CHAR = 1,            /**< Type code representing the SQL type CHAR (1) */
    NUOSQL_VARCHAR = 12,        /**< Type code representing the SQL type VARCHAR (12) */
    NUOSQL_LONGVARCHAR = -1,    /**< Type code representing the SQL type LONGVARCHAR (-1) */
    NUOSQL_DATE = 91,           /**< Type code representing the SQL type DATE (91) */
    NUOSQL_TIME = 92,           /**< Type code representing the SQL type TIME (92) */
    NUOSQL_TIMESTAMP = 93,      /**< Type code representing the SQL type TIMESTAMP (93) */
    NUOSQL_BLOB = 2004,         /**< Type code representing the SQL type BLOB (2004) */
    NUOSQL_CLOB = 2005,         /**< Type code representing the SQL type CLOB (2005) */
    NUOSQL_NUMERIC = 2,         /**< Type code representing the SQL type NUMERIC (2) */
    NUOSQL_DECIMAL = 3,         /**< Type code representing the SQL type DECIMAL (3) */
    NUOSQL_BOOLEAN = 16,        /**< Type code representing the SQL type BOOLEAN (16) */
    NUOSQL_BINARY = -2,         /**< Type code representing the SQL type BINARY (-2) */
    NUOSQL_LONGVARBINARY = -4,   /**< Type code representing the SQL type LONGVARBINARY (-4) */
    NUOSQL_TYPE_UNUSED = 0
};

/**
 * @brief   An object that can be used to get information about the types and 
 *          properties of the columns in a ResultSet object. 
 */
class ResultSetMetaData
{
public:
    virtual ~ResultSetMetaData() = 0;

    /**
     * @brief   Gets the designated column's table name.
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  table name or "" if not applicable
     */
    virtual const char* getTableName(int index) = 0;

    /**
     * @brief   Gets the designated column's name.
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  column name
     */
    virtual const char* getColumnName(int index) = 0;

    /**
     * @brief   Indicates the designated column's normal maximum width in 
     *          characters.
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  the normal maximum number of characters allowed as the width of 
     *          the designated column
     */
    virtual int         getColumnDisplaySize(int index) = 0;

    /**
     * @brief   Retrieves the designated colunmn's SQL type
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  the column's SQL type as defined at NuoDB::SqlType
     */
    virtual int         getColumnType(int index) = 0;

    /**
     * @brief   Retrieves the designated colunmn's type name
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  the columnn type name
     */
    virtual const char* getColumnTypeName(int index) = 0;

    /**
     * @brief   returns the number of columns in this ResultSet object
     *
     * @return  the number of columns
     */
    virtual int         getColumnCount() = 0;

    /**
     * @brief   Get the designated column's specified column size
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  precision
     */
    virtual int         getPrecision(int index) = 0;

    /**
     * @brief   Get the designated column's number of digits to the right of 
     *          the decimal point
     *
     * 0 is returned for data types where the scale is not applicable
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  scale
     */
    virtual int         getScale(int index) = 0;

    /**
     * @brief   Indicates the nullability of values in the designated column
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if the column is nullable and false otherwise 
     */
    virtual bool        isNullable(int index) = 0;

    /* not implemented */
    virtual int         objectVersion() = 0;

    /**
     * @brief   Gets the designated column's suggested title for use in 
     *          printouts and displays. 
     *
     * The suggested title is usually specified by the SQL AS clause.  If a SQL 
     * AS is not specified, the value returned from getColumnLable() will be the
     * same as the value returned by getColumnName()
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  the column label
     */
    virtual const char* getColumnLabel(int index) = 0;

    /**
     * @brief   Indicates whether values in the designated columns are signed numbers
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if signed and false otherwise
     */
    virtual bool        isSigned(int index) = 0;

    /**
     * @brief   Indicates whether the designated column is definitely not writable
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if not writable and false otherwise
     */
    virtual bool        isReadOnly(int index) = 0;

    /**
     * @brief   Indicates whether it is possible for a write on the designated 
     *          column to succeed
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if possibly writable and false otherwise
     */
    virtual bool        isWritable(int index) = 0;

    /**
     * @brief   Indicates whether it is definitely possible for a write on the 
     *          designated column to succeed
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if definitely writable and false otherwise
     */
    virtual bool        isDefinitelyWritable(int index) = 0;

    /**
     * @brief   Indicates whether the designated column is a cash value
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if so and false otherwise
     */
    virtual bool        isCurrency(int index) = 0;

    /**
     * @brief   Indicates whether the designated column is case sensitve
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if so and false otherwise
     */
    virtual bool        isCaseSensitive(int index) = 0;

    /**
     * @brief   Indicates whether the designated column is auto increment
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if so and false otherwise
     */
    virtual bool        isAutoIncrement(int index) = 0;

    /**
     * @brief   Indicates whether the designated column is searchable
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  true if so and false otherwise
     */
    virtual bool        isSearchable(int index) = 0;

    /**
     * @brief   Gets the designated column's table's schema
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  schema name or "" if not applicable
     */
    virtual const char* getSchemaName(int index) = 0;

    /**
     * @brief   Gets the designated column's table's catalog
     *
     * %NuoDB does not support the concept of a catalog
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  ""
     */
    virtual const char* getCatalogName(int index) = 0;

    /**
     * @brief   Gets the maximum column width which would be the larger of the 
     *          length of the value stored in the designated column and the 
     *          declared length of the column
     *
     * @param   index   the first column is 1, the second is 2, ...
     *
     * @return  the max length
     */
    virtual int         getCurrentColumnMaxLength(int column) = 0;
};
}                   // end namespace NuoDB
