// (C) Copyright NuoDB, Inc. 2013-2015  All Rights Reserved.

#pragma once

#include <stdint.h>
#include <time.h>

#include "NuoRemote/DateClass.h"
#include "NuoRemote/SqlExports.h"

namespace NuoDB
{

/**
 * @brief   C++ class type equivalent to SQL DATE
 *
 * Implementation of NuoDB::Date.
 * The date value is always in terms of time since the Epoch,
 * 1970-01-01 00:00:00 +0000 (UTC).
 * Supports only methods related to year, month, and day.
 * utc_time does not account for daylight savings or time zones.
 */
class NUODB_SQLAPI SqlDate : public Date
{
public:
    SqlDate();

    /**
     * @brief   Create a SqlDate as milliseconds since the Epoch,
     *          1970-01-01 00:00:00.000 +0000 (UTC).
     *
     * @param   millis  Number of milliseconds since Epoch
     */
    SqlDate(int64_t millis);

    /**
     * @brief   Create a SqlDate from the date specified in utc_time, which
     *          will be converted to seconds since the Epoch,
     *          1970-01-01 00:00:00 +0000 (UTC).
     *
     * @param   utc_time    C++ standard time struct (<ctime>) representing
     *                      time since Epoch
     */
    SqlDate(struct tm* utc_time);

    ~SqlDate();

public:
    /**
     * @brief   Return the value of the corresponding SQL DATE in milliseconds
     *
     * The milliseconds returned will be UTC milliseconds since the common
     * (UNIX) epoch, Jan 1, 1970.
     *
     * @return  the number of milliseconds in the SQL DATE
     */
    int64_t getMilliseconds();

    /**
     * @brief   Set the Date object with the specified number of milliseconds
     *
     * @param   milliseconds    a 64-bit integer specifying the number of
     *                          milliseconds since the common (UNIX) epoch,
     *                          January 1, 1970
     */
    void setMilliseconds(int64_t millis);

    /**
     * @brief   Return the value of the corresponding SQL DATE in seconds
     *
     * The seconds returned will be UTC seconds since the common (UNIX) epoch,
     * Jan 1, 1970.
     *
     * @return  the number of seconds in the SQL DATE
     */
    int64_t getSeconds();

    /**
     * @brief   Set the Date object with the specified number of seconds
     *
     * @param   seconds     a 64-bit integer specifying the number of seconds
     *                      since the common (UNIX) epoch, January 1, 1970
     */
    void setSeconds(int64_t secs);

    /**
     * @brief   Populate the utc_time struct with the value from this Date
     *          based on the UTC time zone (versus the local time zone)
     *
     * @param   utc_time    C++ standard time struct (<ctime>)
     */
    void getDate(struct tm* utc_time);

    /**
     * @brief   Set this Date from the date specified in utc_time
     *
     * @param   utc_time    C++ standard time struct (<ctime>)
     */
    void setDate(struct tm* utc_time);

    /**
     * @brief   Populate the tz_time struct with the value from this Date
     *          based on the time zone being passed in
     *
     * to UTF-8. %NuoDB Transaction Engines require all strings to be UTF-8.
     * setConverter() allows you to specify a non-default character set table
     * (encoding) that is being used by the client. This may have options appended
     * to the string. IANA alias character set names, IBM CCSIDs starting with
     * "ibm-", Windows codepage numbers starting with "windows-" are frequently '
     * used for this parameter.  See
     * <a href="http://icu-project.org/apiref/icu4c/ucnv_8h.html#a485c78d4149165f504effa2287717e41">
     * <strong>ICU ucnv_open()</strong></a> for more information.
     *
     * @param   timeZoneId  %NuoDB uses ICU for timezone conversions. timezoneId
     *                      is an id for a time zone, such as "America/Los_Angeles",
     *                      or a custom ID such as "GMT-8:00".  It is passed as
     *                      an argument to
     *                      <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param   tz_time     C++ standard time struct (<ctime>)
     *
     */
    void getDate(const char* timeZoneId, struct tm* tz_time);

    /**
     * @brief   Set this date from the time specified in tz_time using the
     *          time zone being passed in
     *
     * @param   timeZoneId  %NuoDB uses ICU for timezone conversions. timezoneId
     *                      is an id for a time zone, such as "America/Los_Angeles",
     *                      or a custom ID such as "GMT-8:00".  It is passed as
     *                      an argument to
     *                      <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param   tz_time     C++ standard time struct (<ctime>)
     *
     */
    void setDate(const char* timeZoneId, struct tm* tz_time);

protected:
    /**
     * @brief   Increment the reference count for this object
     */
    void    addRef();

    /**
     * @brief   Decrement the reference count for this object and if the
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    int     release();

private:
    int64_t millis_;
};
}
