// (C) Copyright NuoDB, Inc. 2013  All Rights Reserved.
#pragma once

#include <stdint.h>
#include <time.h>

#include "NuoRemote/SqlExports.h"
#include "NuoRemote/TimeClass.h"

namespace NuoDB
{
/**
 * @brief   C++ class type equivalent to SQL TIME
 *
 * Implementation of NuoDB::Time.
 * Supports only methods related to hour, minute, and second.
 * utc_time does not account for daylight savings or time zones.
 */
class NUODB_SQLAPI SqlTime : public Time
{
public:
    SqlTime();

    /**
     * @brief   Create a SqlTime as milliseconds since the Epoch,
     *          1970-01-01 00:00:00.000 +0000 (UTC).
     *
     * @param   millis  Number of milliseconds since Epoch
     */
    SqlTime(int64_t millis);

    /**
     * @brief   Create a SqlTime from the date specified in utc_time, which
     *          will be converted to seconds since the Epoch,
     *          1970-01-01 00:00:00 +0000 (UTC).
     *
     * @param   utc_time    C++ standard time struct (<ctime>) representing
     *                      time since Epoch
     */
    SqlTime(struct tm* utc_time);

    ~SqlTime();

public:
    /**
     * @brief   Return the value of the corresponding SQL TIME in milliseconds
     *
     * The milliseconds returned will be UTC milliseconds since the common
     * (UNIX) epoch, Jan 1, 1970.
     *
     * @return  the number of milliseconds in the SQL TIME
     */
    int64_t getMilliseconds();

    /**
     * @brief   Set the Time object with the specified number of milliseconds
     *
     * @param   milliseconds    a 64-bit integer specifying the number of
     *                          milliseconds since the common (UNIX) epoch,
     *                          January 1, 1970
     */
    void setMilliseconds(int64_t millis);

    /**
     * @brief   Return the value of the corresponding SQL TIME in seconds
     *
     * The seconds returned will be UTC seconds since the common (UNIX) epoch,
     * Jan 1, 1970.
     *
     * @return  the number of seconds in the SQL TIME
     */
    int64_t getSeconds();

    /**
     * @brief   Set the Time object with the specified number of seconds
     *
     * @param   seconds     a 64-bit integer specifying the number of seconds
     *                      since the common (UNIX) epoch, January 1, 1970
     */
    void setSeconds(int64_t secs);

    /**
     * @brief   Populate the utc_time struct with the value from this Time
     *          based on the UTC time zone (versus the local time zone)
     *
     * @param   utc_time    C++ standard time struct (<ctime>)
     */
    void getTime(struct tm* utc_time);

    /**
     * @brief   Set this Time from the time specified in utc_time
     *
     * @param   utc_time    C++ standard time struct (<ctime>)
     */
    void setTime(struct tm* utc_time);

    /**
     * @brief   Populate the tz_time struct with the value from this Time
     *          based on the time zone being passed in
     *
     * @param   timeZoneId  %NuoDB uses ICU for timezone conversions. timezoneId
     *                      is an id for a time zone, such as "America/Los_Angeles",
     *                      or a custom ID such as "GMT-8:00".  It is passed as
     *                      an argument to
     *                      <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param   tz_time     C++ standard time struct (<ctime>)
     *
     */
    void getTime(const char* timeZoneId, struct tm* tz_time);

    /**
     * @brief   Set this Time from the time specified in tz_time using the
     *          time zone being passed in
     *
     * @param   timeZoneId  %NuoDB uses ICU for timezone conversions. timezoneId
     *                      is an id for a time zone, such as "America/Los_Angeles",
     *                      or a custom ID such as "GMT-8:00".  It is passed as
     *                      an argument to
     *                      <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param   tz_time     C++ standard time struct (<ctime>)
     *
     */
    void setTime(const char* timeZoneId, struct tm* tz_time);

protected:
    /**
     * @brief   Increment the reference count for this object
     */
    void    addRef();

    /**
     * @brief   Decrement the reference count for this object and if the
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    int     release();

private:
    int64_t millis_;
};
}
