/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

#include "NuoRemote/SqlExports.h"

namespace NuoDB
{
class Connection;
class ResultSet;
class ResultList;
class BigDecimal;
class Date;
class Timestamp;
class Time;
class Clob;
class Blob;
class SQLWarning;

static const int    STATEMENT_VERSION       = 1;
static const int    CLOSE_CURRENT_RESULT    = 1;
static const int    KEEP_CURRENT_RESULT     = 2;
static const int    CLOSE_ALL_RESULTS       = 3;
static const int    SUCCESS_NO_INFO         = -2;
static const int    EXECUTE_FAILED          = -3;

/**
 * Indicates to Statement::execute...() methods that auto generated keys should
 * be available for retrieval.
 */
static const int    RETURN_GENERATED_KEYS   = 1;

/**
 * Indicates to Statement::execute...() methods that auto generated keys need
 * not be available for retrieval.
 */
static const int    NO_GENERATED_KEYS       = 2;

/**
 * @brief   The object used for executing a static SQL statement and returning
 *          the results it produces.
 *
 * Only one ResultSet object per Statement object can be open at the same time.
 * Therefore, if the reading of one ResultSet object is interleaved with the reading
 * of another, each must have been generated by different Statement objects.
 * All execution methods in the Statement class implicitly close a statment's
 * current ResultSet object if an open one exists.
 */
class Statement
{
protected:
    virtual ~Statement() {}

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the
     *          reference count is 0, delete the object
     */
    virtual int     release() = 0;

    /**
     * @brief   Executes the given SQL statement, which returns a single
     *          ResultSet object.
     *
     * Note: This method cannot be called on a PreparedStatement or CallableStatement.
     *
     * @param   sql     an SQL statement to be sent to the database, typically a
     *                  static SQL SELECT statement
     *
     * @return  a ResultSet object that contains the data produced by the given
     *          query; never null
     *
     * @throw   SQLException
     */
    virtual ResultSet*  executeQuery(const char* sql) = 0;

    /**
     * @brief   Executes the given SQL statement, which may be an INSERT, UPDATE,
     *          or DELETE statement or an SQL statement that returns nothing,
     *          such as an SQL DDL statement.
     *
     * Note: This method cannot be called on a PreparedStatement or CallableStatement.
     *
     * @param   sql     an SQL statement to be sent to the database, typically a
     *                  static SQL SELECT statement
     *
     * @return  either (1) the row count for SQL Data Manipulation Language (DML)
     *          statements or (2) 0 for SQL statements that return nothing
     *
     * @throw   SQLException
     */
    virtual int         executeUpdate(const char* sql) = 0;

    /**
     * @brief   Releases this Statement object's database resources and calls
     *          its destructor immediately. It is generally good practice to
     *          release resources as soon as you are finished with them to
     *          avoid tying up database resources.
     *
     * @throw   SQLException
     */
    virtual void        close() = 0;

    /* not implemented */
    virtual int         getMaxFieldSize() = 0;

    /* not implemented */
    virtual void        setMaxFieldSize(int max) = 0;

    /* not implemented */
    virtual int         getMaxRows() = 0;

    /* not implemented */
    virtual void        setMaxRows(int max) = 0;

    /* not implemented */
    virtual void        setEscapeProcessing(bool enable) = 0;

    /**
     * @brief   Retrieves the number of seconds the driver will wait for a
     *          Statement object to execute.
     *
     * If the limit is exceeded, a SQLException is thrown.
     *
     * @return  the current query timeout limit in seconds; zero means there is
     *          no limit
     */
    virtual int         getQueryTimeout() = 0;

    /**
     * @brief   Sets the number of seconds the client will wait for a Statement
     *          object to execute to the given number of seconds.
     *
     * By default there is no limit on the amount of time allowed for a running
     * statement to complete. If the limit is exceeded, an SQLException is thrown.
     * This limit applies to the execute(), executeQuery() and executeUpdate()
     * methods.
     *
     * @param   the new query timeout limit in seconds; zero means there is no
     *          limit
     *
     * @throw   SQLException
     */
    virtual void        setQueryTimeout(int seconds) = 0;

    /* not implemented */
    virtual void        cancel() = 0;

    /* not implemented */
    virtual SQLWarning* getWarnings() = 0;

    /* not implemented */
    virtual void        clearWarnings() = 0;

    /* not implemented */
    virtual void        setCursorName(const char* name) = 0;

    /**
     * @brief   Executes the given SQL statement.
     *
     * The execute() method executes an SQL statement and indicates the form of
     * the first result. You must then use the methods getResultSet() or
     * getUpdateCount() to retrieve the result, and getMoreResults() to move to
     * any subsequent result(s).
     *
     * Note: This method cannot be called on a PreparedStatement or CallableStatement.
     *
     * @param   sql     an SQL statement to be sent to the database
     *
     * @return  true if the first result is a ResultSet object; false if it is
     *          an update count or there are no results
     *
     * @throw   SQLException
     */
    virtual bool        execute(const char* sql) = 0;

    /**
     * @brief   Retrieves the current result as a ResultSet object. This method
     *          should be called only once per result.
     *
     * @return  the current result as a ResultSet object or null if the result
     *          is an update count or there are no more results
     *
     * @throw   SQLException
     */
    virtual ResultSet*  getResultSet() = 0;

    /**
     * @brief   Retrieves the current result as an update count
     *
     * If the result is a ResultSet object or there are no more results, -1 is
     * returned. This method should be called only once per result.
     *
     * @return  the current result as an update count; -1 if the current result
     *          is a ResultSet object or there are no more results
     */
    virtual int     getUpdateCount() = 0;

    /**
     * @brief   Moves to this Statement object's next result, returns true if it
     *          is a ResultSet object, and implicitly closes and frees any current
     *          ResultSet object(s) obtained with the method getResultSet().
     *
     * @return  true if the next result is a ResultSet object; false if it is an
     *          update count or there are no more results
     *
     * @throw   SQLException
     */
    virtual bool    getMoreResults() = 0;

    /* not implemented */
    virtual void    setFetchDirection(int direction) = 0;

    /* not implemented */
    virtual int     getFetchDirection() = 0;

    /* not implemented */
    virtual void    setFetchSize(int rows) = 0;

    /* not implemented */
    virtual int     getFetchSize() = 0;

    /* not implemented */
    virtual int     getResultSetConcurrency() = 0;

    /* not implemented */
    virtual int     getResultSetType() = 0;

    /* not implemented */
    virtual void    addBatch(const char* sql) = 0;

    /* not implemented */
    virtual void    clearBatch() = 0;

    /**
     * @brief   Submits a batch of commands to the database for execution and if
     *          all commands execute successfully, returns an array of update counts.
     *
     * The int elements of the array that is returned are ordered to correspond
     * to the commands in the batch, which are ordered according to the order in
     * which they were added to the batch. The elements in the array returned by
     * the method executeBatch may be one of the following:
     *
     *  1. A number greater than or equal to zero -- indicates that the command was
     *  processed successfully and is an update count giving the number of rows
     *  in the database that were affected by the command's execution
     *
     *  2. A value of SqlCode::PLATFORM_EXCEPTION (-36)  -- indicates that the
     *  command failed to execute successfully and occurs only if a driver
     *  continues to process commands after a command fails.
     *
     * SQLException is thrown if any value in the returned array causes an
     * exception to be thrown. If any statement in the executed batch generates
     * a constraint error, then BatchUpdateException::getText() will return
     * "violation of constraint \"name\"", where name is the name of the constraint.
     *
     * Note: executeBatch() is only implemented in subclasses PreparedStatement
     * and CallableStatement.
     *
     * @return  an array of update counts containing one element for each command
     *          in the batch.  The elements of the array are ordered according
     *          to the order in which commands were added to the batch.
     *
     * @throw   SQLException if one of the commands sent to the database fails
     *          to execute properly or attempts to return a result set.
     */
    virtual int*        executeBatch() = 0;

    /**
     * @brief   Retrieves a pointer to the Connection object that produced this
     *          Statement object.
     *
     * @return  a pointer to the Connection object
     */
    virtual Connection* getConnection() = 0;

    /* not implemented */
    /* c-api-suffix=from */
    virtual bool        getMoreResults(int current) = 0;

    /**
     * @brief   Retrieves any auto-generated keys created as a result of
     *          executing this Statement object.
     *
     * If this Statement object did not generate any keys, an empty ResultSet
     * object is returned.
     *
     * @return  a pointer to a ResultSet object
     *
     * @throw   SQLException
     */
    virtual ResultSet*  getGeneratedKeys() = 0;


    /**
     * @brief   Executes the given SQL statement and signals %NuoDB with the
     *          given flag about whether the auto-generated keys produced by
     *          this Statement object should be made available for retrieval.
     *
     * %NuoDB will ignore the flag if the SQL statement is not an INSERT
     * statement, or some other SQL statement able to return auto-generated
     * keys.
     *
     * Note: This method cannot be called on a PreparedStatement or
     * CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   autoGeneratedKeys   a flag indicating whether auto-generated keys
     *                              should be made available for retrieval;
     *                              one of the following constants:
     *                              NuoDB::RETURN_GENERATED_KEYS
     *                              NuoDB::NO_GENERATED_KEYS
     *                              Any other value will cause an exception to
     *                              be thrown
     *
     * @return  either (1) the row count for SQL Data Manipulation Language (DML)
     *          statements or (2) 0 for SQL statements that return nothing
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_keys */
    virtual int         executeUpdate(const char* sql, int autoGeneratedKeys) = 0;

    /**
     * @brief   Executes the given SQL statement and signals %NuoDB that the
     *          auto-generated keys indicated in the given array should be made
     *          available for retrieval.
     *
     * This array contains the indexes of the columns in the target table that
     * contain the auto-generated keys that should be made available. The array
     * will be ignored if the SQL statement is not an INSERT statement, or some
     * other SQL statement able to return auto-generated keys.
     *
     * Note: This method cannot be called on a PreparedStatement or
     * CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   columnIndexes       an array of column indexes indicating the
     *                              columns that should be returned from the
     *                              inserted row
     * @param   numberIndexes       the number of entries in the columnIndexes
     *                              parameter
     *
     * @return  either (1) the row count for SQL Data Manipulation Language (DML)
     *          statements or (2) 0 for SQL statements that return nothing
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_indexes */
    virtual int         executeUpdate(const char* sql, int* columnIndexes, int numberIndexes) = 0;

    /**
     * @brief   Executes the given SQL statement and signals %NuoDB that the
     *          auto-generated keys indicated in the given array should be made
     *          available for retrieval.
     *
     * This array contains the names of the columns in the target table that
     * contain the auto-generated keys that should be made available. The array
     * will be ignored if the SQL statement is not an INSERT statement, or some
     * other SQL statement able to return auto-generated keys.
     *
     * Note: This method cannot be called on a PreparedStatement or
     * CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   columnNames         an array of column labels indicating the
     *                              columns that should be returned from the
     *                              inserted row
     * @param   numbercolumnNames   the number of entries in the columnNames
     *                              parameter
     *
     * @return  either (1) the row count for SQL Data Manipulation Language (DML)
     *          statements or (2) 0 for SQL statements that return nothing
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_names */
    virtual int         executeUpdate(const char* sql, const char** columnNames, int numberColumnNames) = 0;

    /**
     * @brief   Executes the given SQL statement and signals %NuoDB that any
     *          auto-generated keys should be made available for retrieval.
     *
     * The autoGeneratedKeys flag will be ignored if the SQL statement is not an
     * INSERT statement, or some other SQL statement able to return auto-generated
     * keys.
     *
     * The execute() method executes an SQL statement and indicates the form of
     * the first result. You must then use the methods getResultSet() or
     * getUpdateCount() to retrieve the result, and getMoreResults() to move to
     * any subsequent result(s).
     *
     * Note: This method cannot be called on a PreparedStatement or CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   autoGeneratedKeys   a flag indicating whether auto-generated keys
     *                              should be made available for retrieval;
     *                              one of the following constants:
     *                              NuoDB::RETURN_GENERATED_KEYS
     *                              NuoDB::NO_GENERATED_KEYS
     *                              Any other value will cause an exception to
     *                              be thrown
     *
     * @return  true if the first result is a ResultSet object and false if it
     *          is an update count or there are no results
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_keys */
    virtual bool        execute(const char* sql, int autoGeneratedKeys) = 0;

    /**
     * @brief   Executes the given SQL statement and signals %NuoDB that the
     *          auto-generated keys indicated in the given array should be made
     *          available for retrieval.
     *
     * This array contains the indexes of the columns in the target table that
     * contain the auto-generated keys that should be made available. The array
     * will be ignored if the SQL statement is not an INSERT statement, or some
     * other SQL statement able to return auto-generated keys.
     *
     * Note: This method cannot be called on a PreparedStatement or
     * CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   columnIndexes       an array of column indexes indicating the
     *                              columns that should be returned from the
     *                              inserted row
     * @param   numberIndexes       the number of entries in the columnIndexes
     *                              parameter
     *
     * @return  true if the first result is a ResultSet object and false if it
     *          is an update count or there are no results
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_indexes */
    virtual bool        execute(const char* sql, int* columnIndexes, int numberIndexes) = 0;

    /**
     * @brief   Executes the given SQL statement and signals %NuoDB that the
     *          auto-generated keys indicated in the given array should be made
     *          available for retrieval.
     *
     * This array contains the names of the columns in the target table that
     * contain the auto-generated keys that should be made available. The array
     * will be ignored if the SQL statement is not an INSERT statement, or some
     * other SQL statement able to return auto-generated keys.
     *
     * Note: This method cannot be called on a PreparedStatement or
     * CallableStatement.
     *
     * @param   sql                 an SQL Data Manipulation Language (DML)
     *                              statement, such as INSERT, UPDATE or DELETE;
     *                              or an SQL statement that returns nothing, such
     *                              as a DDL statement.
     * @param   columnNames         an array of column labels indicating the
     *                              columns that should be returned from the
     *                              inserted row
     * @param   numbercolumnNames   the number of entries in the columnNames
     *                              parameter
     *
     * @return  true if the first result is a ResultSet object and false if it
     *          is an update count or there are no results
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_names */
    virtual bool        execute(const char* sql, const char** columnNames, int numberColumnNames) = 0;

    /* not implemented */
    virtual int         getResultSetHoldability() = 0;

    /* not implemented */
    virtual bool        isClosed() = 0;

    /* not implemented */
    virtual void        setPoolable(bool poolable) = 0;

    /* not implemented */
    virtual bool        isPoolable() = 0;

    /* not implemented */
    virtual void        closeOnCompletion() = 0;

    /* not implemented */
    virtual bool        isCloseOnCompletion() = 0;

    /* not implemented */
    virtual ResultList* search(const char* searchString) = 0;

    /* not implemented */
    virtual int         objectVersion() = 0;

    /**
     * @brief   Generates table and index statistics for this command
     *
     * For more information about generating index statistics see
     * <a href="http://doc.nuodb.com/Latest/Default.htm#ANALYZE.htm"><strong>ANALYZE</strong></a>.
     *
     * @param   mask    A bitwise Or including one or both of
     *                  NuoDB::analyzeStatementRecord and
     *                  NuoDB::analyzeStatementExecution
     *
     * @throw SQLException
     */
    virtual const char* analyze(int mask) = 0;

    /* not implemented */
    virtual int getBatchCount() = 0;

    /**
     * @brief   Sets the number of microseconds the client will wait for a
     *          Statement object to execute to the given number of seconds.
     *
     * By default there is no limit on the amount of time allowed for a running
     * statement to complete. If the limit is exceeded, an SQLException is thrown.
     * This limit applies to the execute(), executeQuery() and executeUpdate()
     * methods.
     *
     * @param   the new query timeout limit in seconds; zero means there is no
     *          limit
     *
     * @throw   SQLException
     */
    virtual void        setQueryTimeoutMicros(int64_t microSeconds) = 0;

    /**
     * @brief   Retrieves the number of microseconds the driver will wait for
     *          a Statement object to execute.
     *
     * If the limit is exceeded, a SQLException is thrown.
     *
     * @return  the current query timeout limit in microseconds; zero means there
     *          is *          no limit
     */
    virtual int64_t     getQueryTimeoutMicros() = 0;

    /**
     * @brief  Get server execution time of the statement if it was the last
     * statement executed on the connection. DDL statements are not measured.
     * @return time in microseconds or -1 for DDL statements
     */
    virtual int64_t     getLastStatementTimeMicros() = 0;
};
}                   // end namespace NuoDB
