/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

struct tm;

namespace NuoDB
{

/**
 * @brief   C++ class type equivalent to SQL TIMESTAMP
 *
 * This class will allow you to manipulate this Timestamp object that represents
 * the SQL TIMESTAMP object stored in the database.
 *
 * See also NuoDB::SqlTimestamp.
 */
class Timestamp
{
protected:
    virtual ~Timestamp() = 0;

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the 
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /**
     * @brief   Return the value of the corresponding SQL TIMESTAMP as seconds
     *
     * The seconds returned will be UTC seconds since the common (UNIX) epoch, 
     * Jan 1, 1970.
     *
     * @return  the number of seconds in the SQL TIMESTAMP
     */
    virtual int64_t getSeconds() = 0;   

    /**
     * @brief   Set the Timestamp object with the specified number of seconds
     *
     * @param   seconds     a 64-bit integer specifying the number of seconds 
     *                      since the common (UNIX) epoch, January 1, 1970
     */
    virtual void    setSeconds(int64_t seconds) = 0;

    /**
     * @brief   Return the value of the corresponding SQL TIMESTAMP as milliseconds
     *
     * The milliseconds returned will be UTC milliseconds since the common 
     * (UNIX) epoch, Jan 1, 1970.
     *
     * @return  the number of milliseconds in the SQL TIMESTAMP
     */
    virtual int64_t getMilliseconds() = 0;

    /**
     * @brief   Set the Timestamp object with the specified number of milliseconds
     *
     * @param   milliseconds    a 64-bit integer specifying the number of 
     *                          milliseconds since the common (UNIX) epoch, 
     *                          January 1, 1970
     */
    virtual void    setMilliseconds(int64_t milliseconds) = 0;

    /**
     * @brief   Return the value of the fractional part of the corresponding SQL 
     *          TIMESTAMP as nanoseconds
     *
     * @return  the number of nanoseconds in the fraction of seconds in the SQL 
     *          TIMESTAMP
     */
    virtual int     getNanos() = 0;

    /**
     * @brief   Set the fractions of a second in the Timestamp object with the 
     *          specified number of nanoseconds
     *
     * This does not affect the seconds part of the Timestamp.
     *
     * @param   nanos   a 32-bit integer specifying the number of nanoseconds 
     *                  since the common (UNIX) epoch, January 1, 1970
     */
    virtual void    setNanos(int nanos) = 0;
};
} // end of namespace NuoDB
