/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_CONNECTION_H_
#define _NUODB_CONNECTION_H_ 1

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file connection.h
 *
 * @brief NuoDB Database Connection.
 */

/*! NuoDB Transaction Isolation Levels Enum */
enum NuoDB_TransactionIsolationLevel_enum {
        /**
         * @brief   Read Committed
         *
         * Each transaction always reads the most recently committed
         * version of a record.
         * See <a href="http://doc.nuodb.com/display/doc/Transactions+and+Isolation+Levels"><strong>
         * Transactions and Isolation Levels</strong></a> for more information.
         */
        NUODB_TXN_ISOLATION_READ_COMMITTED  = 2,

        /**
         * @brief   Write Committed
         *
         * For read operations, each transaction sees the state of the
         * database as it was when the transaction started, plus its
         * own changes, similar to CONSISTENT_READ.
         * For write operations, including SELECT...FOR UPDATE, UPDATE
         * and DELETE), each transaction reads the most recently
         * committed version of a record, similar to READ_COMMITTED. See
         * <a href="http://doc.nuodb.com/display/doc/Transactions+and+Isolation+Levels"><strong>
         * Transactions and Isolation Levels</strong></a> for more information.
         */
        NUODB_TXN_ISOLATION_WRITE_COMMITTED  = 5,

        /**
         * @brief   Consistent Read
         *
         * This is the default %NuoDB transaction isolation
         * level. Each transaction sees a snapshot of the database as
         * of the start of the transaction, plus changes made by that
         * transaction only. See
         * <a href="http://doc.nuodb.com/display/doc/Transactions+and+Isolation+Levels"><strong>
         * Transactions and Isolation Levels</strong></a> for more information.
         */
        NUODB_TXN_ISOLATION_CONSISTENT_READ = 7,   /* NuoDB Default */

        /**
         * @brief   Serializable
         *
         * This is kept for backward compatibility and will also cause
         * the same behavior as CONSISTENT_READ which is the default
         * %NuoDB transaction isolation level.
         * Each transaction sees a snapshot of the database as of the
         * start of the transaction, plus changes made by that
         * transaction only. See
         * <a href="http://doc.nuodb.com/display/doc/Transactions+and+Isolation+Levels"><strong>
         * Transactions and Isolation Levels</strong></a> for more information.
         */
        NUODB_TXN_ISOLATION_SERIALIZABLE = 8,
};


/*! NuoDB Transaction Isolation Levels */
typedef enum NuoDB_TransactionIsolationLevel_enum  NuoDB_TransactionIsolationLevel;

/*! NuoDB Database Connection.
 *
 * NuoDB_Connection is a data structure that represents a connection
 * to a NuoDB Database.  Clients should call the NuoDB_Connection_create()
 * function to create this data structure and then call the openDatabase()
 * function to open a connection to a %NuoDB database.  Clients should
 * call the NuoDB_Connection_free() function to release the
 * NuoDB_Connection when the client application no longer needs the
 * connection.
 */
struct NuoDB_Connection
{
    /** @internal */
    _NuoDB_Connection_state_t* _state;

    /**
     * @fn commit()
     *
     * @brief   Commit any outstanding SQL statements.
     *
     * Makes all changes made since the previous commit/rollback
     * permanent and releases any database locks currently held by
     * this NuoDB_Connection data structure. This function should be
     * used only when auto-commit mode has been disabled.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* commit)(NuoDB_Connection* connection);

    /**
     * @brief   Convert the given, presumably UTF-8 string, to the
     *          local encoding using the NuoDB_Connection's active
     *          encoding. See:
     *          NuoDB_Connection::getActiveEncoding().
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]   s            a pointer to the string.
     * @param[in]   sLen         length of the string.
     *
     * @return  a pointer to the converted string or NULL if there was
     * an error.  The client is responsible for freeing this memory.
     */
    char* (* convertUtf8ToLocal)(NuoDB_Connection* connection, const char* s, size_t sLen);

    /**
     * @brief   Return the name of the active encoding for this
     *          NuoDB_connection which may not be the same as the
     *          default encoding for this NuoDB_Connection.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  a string that contains the name of the active encoding.
     */
    const char*     (* getActiveEncoding)(NuoDB_Connection* connection);

    /**
     * @brief   Determine if the NuoDB_Connection has the auto-commit
     * flag set.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[out]  autoCommit   pointer to nuodb_bool_t to set
     * auto-commit value.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* getAutoCommit)(NuoDB_Connection* connection, nuodb_bool_t* autoCommit);

    /**
     * @brief   Returns the connection version
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  the connection version
     */
    int (* getConnectionVersion)(NuoDB_Connection* connection);

    /**
     * @brief   Return the name of the default encoding for this
     * NuoDB_Connection.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  the name of the default encoding
     */
    const char* (* getDefaultEncoding)(NuoDB_Connection* connection);

    /**
     * @brief   Retrieve metadata for the database associated to this
     *          NuoDB_Connection.
     *
     * The metadata includes information about the database's tables, its
     * supported SQL grammar, its stored procedures, the capabilities
     * of this NuoDB_Connection, and so on.
     *
     * The NuoDB_Connection functions manage the memory allocated in
     * the DatabaseMetaData data structure and the caller does not
     * have to do so.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  pointer to a NuoDB_DatabaseMetaData data structure or
     * NULL on error.
     */
    NuoDB_DatabaseMetaData* (* getMetaData)(NuoDB_Connection* connection);

    /**
     * @brief   Retrieves this NuoDB_Connection data structure's
     *          current transaction isolation level
     *
     * The transction isolation level will be one of:
     * <ul>
     * <li>NUODB_TXN_ISOLATION_WRITE_COMMITTED = 5</li>
     * <li>NUODB_TXN_ISOLATION_READ_COMMITTED  = 2</li>
     * <li>NUODB_TXN_ISOLATION_CONSISTENT_READ = 7</li>
     * <li>NUODB_TXN_ISOLATION_SERIALIZABLE    = 8</li>
     * </ul>
     *
     * @param[in]   connection      a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[out]  isolationLevel  a pointer to a
     * NuoDB_TransactionIsolationLevel.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* getTransactionIsolation)(NuoDB_Connection* connection, NuoDB_TransactionIsolationLevel* isolationLevel);

    /**
     * @brief   Determine if the client has a valid connection to a
     * %NuoDB TE.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  true if the socket which communicates to the server
     *          exists and is open.
     */
    nuodb_bool_t    (* isConnected)(NuoDB_Connection* connection);

    /**
     * @brief   Retrieves whether this NuoDB_Connection data structure
     * is in read-only mode.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  true if the database is read-only, otherwise returns
     * false.
     */
    nuodb_bool_t    (* isReadOnly)(NuoDB_Connection* connection);

    /**
     * @brief   Retrieves whether this NuoDB_Connection data structure
     * is using UTF-8 strings.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  returns true if the NuoDB_Connection is using UTF-8
     * strings. Otherwise returns false.
     */
    nuodb_bool_t    (* isUtf8)(NuoDB_Connection* connection);

    /**
     * @brief       Open a database.
     *
     * Given an already existing NuoDB_Connection data structure,
     * clients must call this function to open a database.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]   dbname       a pointer to a const char string
     * representing the database name.
     * @param[in]   user         a pointer to a const char string that
     * contains the database username.
     * @param[in]   password     a pointer to a const char string that
     * contains the database password.
     * @param[in]   options      a pointer to optional NuoDB_Options. If
     * no specific options are needed, the caller can pass a NULL.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* openDatabase)(NuoDB_Connection* connection,
                         const char* dbname,
                         const char* user,
                         const char* password,
                         NuoDB_Options* options);
    /**
     * @brief   Send a ping message to the server. If no error occurs,
     * the server is alive.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* ping)(NuoDB_Connection* connection);

    /**
     * @brief   Roll back any outstanding SQL statements.
     *
     * Undoes all changes made in the current transaction and releases
     * any database locks currently held by this NuoDB_Connection
     * data structure. This function should be used only when
     * auto-commit mode has been disabled.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* rollback)(NuoDB_Connection* connection);

    /**
     * @brief   Specify if statements related to this NuoDB_Connection
     *          should be auto-committed.
     *
     * If a NuoDB_Connection is in auto-commit mode, then all its SQL
     * statements will be executed and committed as individual
     * transactions. Otherwise, its SQL statements are grouped into
     * transactions that are terminated by a call to either the
     * function commit() or the function rollback(). By default, new
     * NuoDB_Connections are in auto-commit mode.
     *
     * @param[in]   autoCommit  If true, set autoCommit on and if false, turn
     * autoCommit off
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setAutoCommit)(NuoDB_Connection* connection, nuodb_bool_t autoCommit);

    /**
     * @brief   Puts this NuoDB_Connection in read-only mode as a hint
     *          to enable database optimizations.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]   readOnly     a boolean that indicates the
     * read-only mode.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setReadOnly)(NuoDB_Connection* connection,
                        nuodb_bool_t readOnly);


    /**
     * @brief   Change the transaction isolation level for this
     *          NuoDB_Connection data structure to the one specified.
     *
     * Valid transaction isolation levels are:
     * <ul>
     * <li>NUODB_TXN_ISOLATION_WRITE_COMMITTED = 5</li>
     * <li>NUODB_TXN_ISOLATION_READ_COMMITTED  = 2</li>
     * <li>NUODB_TXN_ISOLATION_CONSISTENT_READ = 7</li>
     * <li>NUODB_TXN_ISOLATION_SERIALIZABLE    = 8</li>
     * </ul>
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setTransactionIsolation)(NuoDB_Connection* connection,
                NuoDB_TransactionIsolationLevel level);

    /**
     * @brief   Specify a non-default encoding for this client
     * NuoDB_Connection.
     *
     * %NuoDB uses ICU (International Components for Unicode) for converting
     * strings from the local client environment to UTF-8. %NuoDB Transaction
     * Engines require all strings to be UTF-8. setConverter() allows you to
     * specify a non-default character set table (encoding) that is being used
     * by the client. This may have options appended to the string. IANA alias
     * character set names, IBM CCSIDs starting with "ibm-", Windows codepage
     * numbers starting with "windows-" are frequently ' used for this
     * parameter.  See
     * <a href="http://icu-project.org/apiref/icu4c/ucnv_8h.html#a485c78d4149165f504effa2287717e41">
     * <strong>ICU ucnv_open()</strong></a> for more information.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]   encoding    Name of a non-default character set table.
     * If this parameter is NULL, the default converter will be used.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setConverter)(NuoDB_Connection* connection,
                         const char* encoding); /* ICU name of non-default character set table */

    /**
     * @brief   Specify a non-default encoding for this client
     *          NuoDB_Connection and set it as the default for the
     *          connection.
     *
     * %NuoDB uses ICU (International Components for Unicode) for converting
     * strings from the local client environment to UTF-8. %NuoDB Transaction
     * Engines require all strings to be UTF-8. setDefaultConverter()
     * allows you to specify a non-default character set table
     * (encoding) that is being used by the client. This may have
     * options appended to the string. IANA alias character set names,
     * IBM CCSIDs starting with "ibm-", Windows codepage numbers
     * starting with "windows-" are frequently used for this
     * parameter. See
     * <a href="http://icu-project.org/apiref/icu4c/ucnv_8h.html#a485c78d4149165f504effa2287717e41">
     * <strong>ICU ucnv_open()</strong></a> for more information.
     *
     * setDefaultConverter() will set the NuoDB_Connections default
     * converter to that for the given encoding. Future calls to
     * getDefaultEncoding() will return this new default encoding.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @param[in]   encoding    Name of a non-default character set table.
     *                      If this parameter is NULL, the default converter
     *                      will be used.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setDefaultConverter)(NuoDB_Connection* connection,
                                const char* encoding);  /* ICU name of non-default character set table */

    /** @internal */
    int (* getProtocolVersion)(NuoDB_Connection* connection);


    /**
     * @brief   Returns the connection URL string
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return  a pointer to the connection string.
     */
    const char* (* getConnectionURL)(NuoDB_Connection* connection);

    /**
     * @brief       Open a database with an NuoEngine.
     *
     * Given an already existing NuoDB_Connection data structure,
     * clients may call this to open a database.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]   nuoEngine    a pointer to an NuoEngine.
     * @param[in]   user         a pointer to a const char string that
     * contains the database username.
     * @param[in]   password     a pointer to a const char string that
     * contains the database password.
     * @param[in]   options      a pointer to optional NuoDB_Options. If
     * no specific options are needed, the caller can pass a NULL.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* openDatabaseEngine)(NuoDB_Connection* connection,
                         void* nuoEngine,
                         const char* user,
                         const char* password,
                         NuoDB_Options* options);

    /**
     * @brief   Returns next sequence value
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     * @param[in]  schema a pointer to a char for database schema
     * @param[in]  sequenceName a pointer to a char for sequence name
     * @param[out] sequence a pointer to a int64_t for retrieved sequence value
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* getNextSequenceValue)(NuoDB_Connection* connection, const char* schema, const char* sequenceName, int64_t* sequence);

    /**
     * @brief       Get a handle to the NuoDB Engine.
     *
     * Return the NuoDBEngine handle, or NULL if not available.
     *
     * @param[in]   connection   a pointer to the current (this)
     * NuoDB_Connection data structure.
     *
     * @return pointer to handle on success, NULL on fialure
     */
    NuoDB_EngineHandle* (* getEngineHandle)(NuoDB_Connection* connection);
};

/**
 * @brief  Create a NuoDB_Connection to the %NuoDB database.
 *
 * In order to perform operations on a %NuoDB database, you need to
 * create a NuoDB_Connection by calling NuoDB_Connection_create(), and
 * then call the NuoDB_Connection::openDatabase() function to
 * establish a connection to the specified database.
 *
 * When clients are finished with the database connection, they should
 * call NuoDB_Connection_free() to release the resources held by the
 * NuoDB_Connection.
 *
 * @return  a pointer to a NuoDB_Connection data structure or NULL if
 * there was any error.
 */
NUODB_CAPI NuoDB_Connection* NUODB_CAPICALL NuoDB_Connection_create();

/**
 * @brief   Create a NuoDB_Connection to the %NuoDB database with
 * UTF-8 string encoding.
 *
 * This overloading will assumes strings being passed from the client
 * to the %NuoDB TE will be UTF-8 despite the encoding set in the
 * user's local environment.
 *
 * @return  a pointer to a NuoDB_Connection data structure or NULL if
 * there was any error.
 */

NUODB_CAPI NuoDB_Connection* NUODB_CAPICALL NuoDB_Connection_createUtf8();

/**
 * @brief   Frees the connection to the database
 *
 * Clients should call NuoDB_Connection_free() on the NuoDB_Connection
 * data structure when the client is finished with the specified
 * NuoDB_Connection.
 *
 * @param[in]   connection   a pointer to the current (this) NuoDB_Connection
 * data structure.
 *
 */
NUODB_CAPI void NUODB_CAPICALL              NuoDB_Connection_free(NuoDB_Connection* connection);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of: #ifndef _NUODB_CONNECTION_H_ */
