/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_OPTIONS_H_
#define _NUODB_OPTIONS_H_ 1

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file options.h
 * @brief Container for NuoDB_Connection options.
 */

/*! Container for NuoDB_Connection options.
 *
 * NuoDB_Options is a simple container that is used to hold a list of
 * NuoDB_Connection options.  Clients should call the
 * NuoDB_Options_create() function to create this container.  Clients
 * can add name/value pairs to the NuoDB_Options container and then
 * pass the NuoDB_Options to the NuoDB_Connection::openDatabase()
 * function.  Clients should call the NuoDB_Options_free() function to
 * release the container where the client application no longer needs
 * the options.
 */
struct NuoDB_Options
{
    /** @internal */
    _NuoDB_Options_state_t* _state;

    /**
     * @brief   Add an option to the list of options.
     *
     * @param[in]   _this  a pointer to the current "this" data structure.
     * @param[in]   name   a pointer to the option name.
     * @param[in]   value  a pointer to the option value.
     *
     * Add the specified name/value pair to the list of options.  If
     * the specified name exists in the list, then this function will
     * replace the previous value.
     */
    void (* add)(NuoDB_Options* _this, const char* name, const char* value);

    /**
     * @brief   Retrieve an option value.
     *
     * This function looks for the specified option name.  If the
     * option name is found, then this function will return a
     * pointer to the value.  If the name is not found, then this
     * function will return a NULL pointer.
     *
     * @param[in]   _this  a pointer to the current "this" data structure.
     * @param[in]   name   a pointer to the option name.
     *
     * @return  a pointer to the value or NULL if the value does not
     * exist.
     */
    const char* (* getValue)(NuoDB_Options* _this, const char* name);

    /**
     * @brief   Clear the options list.
     *
     * This function will remove all name/value pairs from the
     * specified NuoDB_Options container.
     *
     * @param[in]   _this  a pointer to the current "this" data structure.
     */
    void    (* clear)(NuoDB_Options* _this);

    /**
     * @brief   Copy options.
     *
     * This function will copy all of the options from the src
     * container to the current (this) container.
     *
     * @param[in]   _this  a pointer to the current "this" data structure.
     * @param[in]   src    a pointer to the source options.
     */
    void    (* copy)(NuoDB_Options* _this, const NuoDB_Options* src);

    /**
     * @brief   Retrieve a count of the options.
     *
     * This function counts the number of options in the specified
     * NuoDB_Options data structure.
     *
     * @param[in]   _this  a pointer to the current "this" data structure.
     *
     * @return  a count of the options or zero (0) if the container is
     * empty.
     */
    int     (* getCount)(NuoDB_Options* _this);
};

/**
 * @brief   Create a NuoDB_Options container.
 *
 * This is a factory function for creating NuoDB_Options.
 * NuoDB_Options are a container that holds a list NuoDB Database
 * connection options.  Clients should call the NuoDB_Options_free()
 * function to release the resources held by the NuoDB_Options
 * data structure.
 */
NUODB_CAPI NuoDB_Options*  NUODB_CAPICALL   NuoDB_Options_create();


/**
 * @brief   Free NuoDB_Options container.
 *
 * This function is used to free the resources held by the specified
 * NuoDB_Options container.  The client must no use the NuoDB_Options
 * data structure after it has been freed.  The NuoDB_Options data
 * structure can be freed right after it is used by the
 * NuoDB_Connection::openDatabase() function.
 *
 * @param[in]   ops  a pointer to the NuoDB_Options data structure.
 */
NUODB_CAPI void NUODB_CAPICALL              NuoDB_Options_free(NuoDB_Options* ops);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of: #ifndef _NUODB_OPTIONS_H_ */
