/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_STATEMENT_H_
#define _NUODB_STATEMENT_H_ 1

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file statement.h
 * @brief The data structure for executing a SQL statement.
 */

/*! Enum defining constant flags used by NuoDB_Statement. */
enum NuoDB_Statement_Flags_enum {
        /**
         * @brief Bitfield indicates there are NO optional flags
         */
        NUODB_NO_FLAGS          = 0x0,

        /**
         * @brief Bitfield flag for statements with automatically
         * generated keys.
         */
        NUODB_AUTOGENERATEDKEYS = 0x1,

        /**
         * @brief Bitfield flag for statements that execute a query.
         */
        NUODB_EXECUTEQUERY      = 0x2,

        /**
         * @brief Bitfield flag for statements that update.
         */
        NUODB_EXECUTEUPDATE     = 0x4,
};

struct NuoDB_BatchResult
{
    int result;
    const char* errorString;
    const char* stateCode;
};

/*! Constant flags used by NuoDB_Statement. */
typedef enum NuoDB_Statement_Flags_enum  NuoDB_Statement_Flags;


/*! The data structure for executing a SQL statement.
 *
 * NuoDB_Statement is a data structure used for executing a static SQL
 *          statement and returning the results it
 *          produces. NuoDB_Statement supports both parameterized and
 *          non-parameterized SQL statements.
 *
 * Only one NuoDB_ResultSet data structure per NuoDB_Statement data
 * structure can be open at the same time.  Therefore, if the reading
 * of one NuoDB_ResultSet data structure is interleaved with the
 * reading of another, each must have been generated by different
 * NuoDB_Statement data structures. All execution function in the
 * NuoDB_Statement data structure implicitly close and free a
 * statement's current NuoDB_ResultSet data structure if an open one
 * exists.
 * A prepared statement may have a NuoDB_ResultSetMetaData data structure
 * if the statement is a query. Only one NuoDB_ResultSetMetaData data structure
 * can be open from a NuoDB_Statement object. The NuoDB_ResultSetMetaData data structure
 * is implicitly closed and freed when the NuoDB_Statement data structure is released.
*/
struct NuoDB_Statement
{
    /** @internal */
    _NuoDB_Statement_state_t* _state;

    /**
     * @brief Executes a non-parameterized SQL statement.
     *
     * Clients call this function to immediately execute a
     * non-parameterized SQL statement.
     *
     * @param[in]   _this  a pointer to the current (this) data structure.
     * @param[in]   sql    a pointer to a const char* that contains the
     * SQL statement string.
     * @param[in]   flags  a set of OR'd NuoDB_Statement_Flags values.
     * Clients can pass a zero to indicate there are no
     * options. Clients can provide the NUODB_AUTOGENERATEDKEYS
     * bitfield to indicate that the statement should return any auto
     * generated keys.  Clients can provide the NUODB_EXECUTEQUERY
     * bitfield to optimize statements that are expected to return
     * result sets.  Client can provide the NUODB_EXECUTEUPDATE
     * bitfield to optomize statements that are expected to update the
     * database.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* executeSQL)(NuoDB_Statement* _this, const char* sql, unsigned int flags);


    /**
     * @brief   Retrieves any auto-generated keys created as a result of
     *          executing this NuoDB_Statement data structure.
     *
     * If this NuoDB_Statement data structure did not generate any keys, an
     * empty NuoDB_ResultSet data structure is returned.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  a pointer to a NuoDB_ResultSet data structure, or NULL to
     * indicate an error.
     */
    NuoDB_ResultSet*    (* getGeneratedKeys)(NuoDB_Statement* _this);

    /**
     * @brief   Retrieves the current result as a NuoDB_ResultSet data
     *          structure. This function should be called only once
     *          per result.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the current result as a NuoDB_ResultSet data structure or
     * NULL if there are no more results.
     */
    NuoDB_ResultSet*    (* getResultSet)(NuoDB_Statement* _this);

    /**
     * @brief   Clears the current parameter values immediately.
     *
     * In general, parameter values remain in force for repeated use
     * of a statement. Setting a parameter value automatically removes
     * its previous value. However, in some cases it is
     * useful to immediately release the resources used by the current
     * parameter values; this can be done by calling the function
     * clearParameters().
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* clearParameters)(NuoDB_Statement* _this);

    /**
     * @brief Returns the NuoDB_Connection for this statement.
     */
    NuoDB_Connection*   (* getConnection)(NuoDB_Statement* _this);

   /**
     * @brief   Moves to this NuoDB_Statement data structure's next result,
     *          returns the new NuoDB_ResultSet and implicitly closes
     *          and frees any current NuoDB_ResultSet data structure(s)
     *          obtained with the function getResultSet().
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the next NuoDB_ResultSet data structure or NULL
     * if there are no more results.
     */
    NuoDB_ResultSet*    (* getMoreResults)(NuoDB_Statement* _this);

    /**
     * @brief   Retrieves the number of microseconds the driver will
     *          wait for a NuoDB_Statement data structure to execute.
     *
     * If the limit is exceeded, the execute() function will return an
     * error code code.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the current query timeout limit in microseconds; zero
     *          means there is no limit; negative numbers are error codes.
     */
    int64_t (* getQueryTimeout)(NuoDB_Statement* _this);


   /**
    * @brief   Retrieves the update count
    *
    * @param[in]   _this   a pointer to the current (this) data structure.
    *
    * @return  the number of rows updated by the current statement.
    * Returns zero if there were no updates.
    */
   int (* getUpdateCount)(NuoDB_Statement* _this);

    /**
     * @brief   Sets the number of microseconds the client will wait
     *          for a NuoDB_Statement data structure to execute to the
     *          given number of microseconds.
     *
     * By default there is no limit on the amount of time allowed for
     * a running statement to complete. If the limit is exceeded, the
     * execute function will return an error code code.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   the new query timeout limit in microseconds; zero
     * means there is no limit.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setQueryTimeout)(NuoDB_Statement* _this, int64_t microSeconds);

    /**
     * @brief   Prepares a parameterized SQL statement.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   sql                 a pointer to a const char* that
     * contains a valid, optionally parameterized, SQL statement.
     * @param[in]   flags  a set of OR'd NuoDB_Statement_Flags values.
     * Clients can pass a zero to indicate there are no options.
     * Clients can provided the NUODB_AUTOGENERATEDKEY bitfield to
     * indicate that the statement should return any auto generated
     * keys.  Clients can provide the NUODB_EXECUTEQUERY bitfield to
     * optimize statements that are expected to return result sets.
     * Client can provide the NUODB_EXECUTEUPDATE bitfield to optimize
     * statements that are expected to update the database.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* prepare)(NuoDB_Statement* _this, const char* sql, unsigned int flags);

    /**
     * @brief   Executes a previously prepared SQL statement.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* execute)(NuoDB_Statement* _this);

    /**
     * @brief   Sets the designated parameter to the given const char*
     * value.
     *
     * setString() can only be used with nul terminated strings that
     * do not contain any embedded nul characters.  If your
     * application allows strings to have embedded nul characters, or
     * if your strings are not nul terminated, then your client should
     * call setStringWithLength() to explicitly set the length of the
     * stringValue.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index        the first parameter is 1, the second is
     * 2, ...
     * @param[in]   stringValue  the value to which we will set the
     * parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setString)(NuoDB_Statement* _this, int index, const char* stringValue);

    /**
     * @brief   Sets the designated parameter to the given const char* value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index        the first parameter is 1, the second is
     * 2, ...
     * @param[in]   stringValue  the value to which we will set the
     * parameter.
     * @param[in]   length       the length of the stringValue.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setStringWithLength)(NuoDB_Statement* _this, int index, const char* stringValue, size_t length);

    /**
     * @brief   Sets the designated parameter to the given 32-bit
     * integer value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value  the 32-bit integer value to which we will set
     * the parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setInt)(NuoDB_Statement* _this, int index, int32_t value);

    /**
     * @brief   Sets the designated parameter to the given 64-bit
     * integer value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value  the 64-bit integer value to which we will set
     * the parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setLong)(NuoDB_Statement* _this, int index, int64_t value);

    /**
     * @brief   Sets the designated parameter to the given Lob value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value  the NuoDB_Lob value to which we will set the
     * parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setLob)(NuoDB_Statement* _this, int index, NuoDB_Lob* lob);

    /**
     * @brief   Sets the designated parameter to the given boolean value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value  the boolean value to which we will set the
     * parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setBoolean)(NuoDB_Statement* _this, int index, nuodb_bool_t value);

    /**
     * @brief   Sets the designated parameter to the given short value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value  the short value to which we will set the
     * parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setShort)(NuoDB_Statement* _this, int index, short value);

     /**
     * @brief   Sets the designated parameter to the given char value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value   the one-byte value to which we will set the
     * parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
   NuoDB_Status (* setByte)(NuoDB_Statement* _this, int index, char value);

    /**
     * @brief   Sets the designated parameter to SQL NULL.
     *
     * Note: You must specify the parameter's SQL type.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   type    the type code as defined in NUODB_TYPE_* values.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setNull)(NuoDB_Statement* _this, int index, int type);

    /**
     * @brief   Sets the designated parameter to the given
     * double-precision value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value   the double-precision value to which we will
     * set the parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setDouble)(NuoDB_Statement* _this, int index, double value);

    /**
     * @brief   Sets the designated parameter to the given
     * NuoDB_Temporal value.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   index   the first parameter is 1, the second is 2, ...
     * @param[in]   value   the NuoDB_Temporal value to which we will set
     * the parameter.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setTemporal)(NuoDB_Statement* _this, int index, NuoDB_Temporal* timeValue);

    /**
     * @brief   Stores the current set of parameter values as a batch entry
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* addBatch)(NuoDB_Statement* _this);

    /**
     * @brief   Clear the stored batch entries
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* clearBatch)(NuoDB_Statement* _this);

    /**
     * @brief   Execute the batch operations added via addBatch
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[out]  results on output, an array of integer holding the result of the
     *                      corresponding operation
     * @param[out]  resultSize on output, the length of the array whose pointer has
     *                         been placed in the results arguments
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* executeBatch)(NuoDB_Statement* _this, NuoDB_BatchResult** results, size_t* resultSize);

    /**
     * @brief  Setup of the fetch size
     *
     * @param[in]   _this        a pointer to the current (this) data structure.
     * @param[in]   fetchSize    the value of fetch size
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setFetchSize)(NuoDB_Statement* _this, int fetchSize);

    /**
     * @brief   Retrieves result set metadata as a NuoDB_ResultSetMetaData
     *          data structure for a previously prepared statement
     *          which has not been executed yet.
     *          
     *          This function should only be called on a prepared statement before
                its execution.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the result set metadata for a previously prepared statement
     * which has not been executed yet. The prepared statement should be a query (select).
     * NULL if the previously preared statement is not a query or it the statement was
     * not previously prepared.
     */
    NuoDB_ResultSetMetaData*    (* getMetaData)(NuoDB_Statement* _this);
};

/**
 * @brief Creates a NuoDB_Statement data structure.
 *
 * This is a factory function for creating NuoDB_Statements from the
 * specified NuoDB_Connection.  Clients are expected to call the
 * NuoDB_Statement_free() function which the client is finished with
 * statement.
 */
NUODB_CAPI NuoDB_Statement* NUODB_CAPICALL NuoDB_Statement_create(NuoDB_Connection* connection);

/**
 * @brief Frees a NuoDB_Statement data structure.
 *
 * Clients should call this function when they are finished with
 * statement.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Statement_free(NuoDB_Statement* statement);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of: #ifndef _NUODB_STATEMENT_H_ */
