import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;


/** An example program for connecting to a NuoDB database server. */
public class HelloDB {

    /** The driver class provided by NuoDB. */
    public static final String DRIVER_CLASS =
	"com.nuodb.jdbc.Driver";

    /** The base URL for connecting to a local database server. */
    public static final String DATABASE_URL =
	"jdbc:com.nuodb://localhost/";

    // the established connection to a local server
    private final Connection dbConnection;

    /**
     * Creates an instance of HelloDB and connects to a local server,
     * as the given user, to work with the given named database
     *
     * @param user the user name for the connection
     * @param password the password for the given user
     * @param dbName the name of the database at the server to use
     */
    public HelloDB(String user, String password, String dbName)
	throws SQLException
    {
	Properties properties = new Properties();
	properties.put("user", user);
	properties.put("password", password);
	properties.put("schema", "hello");

	dbConnection =
	    DriverManager.getConnection(DATABASE_URL + dbName, properties);
    }

    /** Closes the connection to the server. */
    public void close() throws SQLException {
	dbConnection.close();
    }

    /** Creates a simple three-column table: id-name-balance. */
    public void createAccountTable() throws SQLException {
	try (Statement stmt = dbConnection.createStatement()) {
	    stmt.execute("CREATE TABLE accounts (id int primary key, name string)");
	    stmt.execute("ALTER TABLE accounts ADD COLUMN balance INT");
	    dbConnection.commit();
	}
	catch(Exception exception) {
	    System.out.println("Skipping table creation: " + exception.getMessage());
	}
    }
    
    /**
     * Populate Accounts table with some sample data.
     * @throws SQLException
     */
    public void populateDemo() throws SQLException {
	int[] idArray = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
	 
        String[] nameArray = {"Sarah", "Carl", "Sophia", "John", "Maya", "Will", "Laura",
                    "David", "Lily", "Chris", "Emily", "Matt", "Zoe", "Dan", "Mia"};
 
        int[] accountBalance = {15000, 70000, 8000, 2000, 90000, 40000, 100000,
                                  20000, 11000, 55000, 63000, 10000, 1500, 6000, 47000};
 
        try (PreparedStatement stmt = dbConnection.
        	prepareStatement("insert into accounts (id, name, balance) values (?, ?, ?)")) {
            for (int i = 1; i <=idArray.length; i++) {
                stmt.setInt(1, idArray[i-1]);
                stmt.setString(2, nameArray[i-1]);
                stmt.setInt(3, accountBalance[i-1]);
                stmt.addBatch();              
            }
            stmt.executeBatch(); 
        } catch(Exception exception) {
            System.out.println("Skipping populateDemo..." + exception.getMessage());
        }
        dbConnection.commit();
    }

    /**
     * Inserts a new name into the table with a 0 balance. The id must be unique.
     *
     * @param id a unique numeric identifier
     * @param name a name associated with the given id
     */
    public void insertName(int id, String name) throws SQLException {
	try (PreparedStatement stmt = dbConnection.
		prepareStatement("insert into accounts (id, name, balance) values (?, ?, 0)")) {
	    stmt.setInt(1, id);
	    stmt.setString(2, name);
	    stmt.addBatch();
	    stmt.executeBatch();
	    dbConnection.commit();
	} catch(Exception exception) {
	    System.out.println("Skipping insert..." + exception.getMessage());
	}
    }

    /**
     * Gets the name for the given id, or null if no name exists.
     *
     * @param id an identifier
     * @return the name associated with the identifier, or null
     */
    public String getName(int id) throws SQLException {
	try (PreparedStatement pst = dbConnection.
		prepareStatement("select name from accounts where id=?")) {
	    pst.setInt(1, id);
	    try (ResultSet rs = pst.executeQuery()){
		if (rs.next())
		    return rs.getString(1);
		return null;
	    }   
	}
    }
    
    /**
     * Gets the balance for the given id, or null if no name exists.
     *
     * @param id an identifier
     * @return the balance associated with the identifier, or null
     */
    public String getBalance(int id) throws SQLException {
        try (PreparedStatement pst = dbConnection.
        	prepareStatement("select balance from accounts where id=?")) {
            pst.setInt(1, id);
            try (ResultSet rs = pst.executeQuery()){
                if (rs.next())
                    return rs.getString(1);
                return null;
            }
        }
    }

    /** Main-line for this example. */
    public static void main(String [] args) throws Exception {
	Class.forName(DRIVER_CLASS);
	 
        HelloDB helloDB = new HelloDB("dba", "goalie", "test");
        helloDB.createAccountTable();
        helloDB.populateDemo();
        String name, balance;
 
        for(int i=1; i<16; i++) {
            name = helloDB.getName(i);
            balance = helloDB.getBalance(i);
            if (name != null || balance != null) {
        	System.out.println(name + ":\n Account ID = " + i +
        	    "\n Balance = $" + helloDB.getBalance(i) + "\n");
            } else {
        	System.out.println("Account ID "+i+": NOT FOUND");
            }
        }
        helloDB.close();
    }

}
